@@ -34,9 +34,12 @@
  * the terms of any one of the MPL, the GPL or the LGPL.
  *
  * ***** END LICENSE BLOCK ***** */
+
+/* rendering object for list-item bullets */
+
 #include "nsCOMPtr.h"
 #include "nsBulletFrame.h"
-#include "nsHTMLAtoms.h"
+#include "nsGkAtoms.h"
 #include "nsHTMLParts.h"
 #include "nsHTMLContainerFrame.h"
 #include "nsIFontMetrics.h"
@@ -44,34 +47,37 @@
 #include "nsPresContext.h"
 #include "nsIPresShell.h"
 #include "nsIDocument.h"
-#include "nsReflowPath.h"
 #include "nsIRenderingContext.h"
 #include "nsILoadGroup.h"
 #include "nsIURL.h"
 #include "nsNetUtil.h"
-#include "nsLayoutAtoms.h"
 #include "prprf.h"
-#ifdef IBMBIDI
-#include "nsBidiPresUtils.h"
-#endif // IBMBIDI
+#include "nsDisplayList.h"
 
 #include "imgILoader.h"
 #include "imgIContainer.h"
-#include "imgIDecoderObserver.h"
+#include "nsStubImageDecoderObserver.h"
 
 #include "nsIServiceManager.h"
 #include "nsIComponentManager.h"
 #include "nsContentUtils.h"
 
-class nsBulletListener : public imgIDecoderObserver
+class nsBulletListener : public nsStubImageDecoderObserver
 {
 public:
   nsBulletListener();
   virtual ~nsBulletListener();
 
   NS_DECL_ISUPPORTS
-  NS_DECL_IMGIDECODEROBSERVER
-  NS_DECL_IMGICONTAINEROBSERVER
+  // imgIDecoderObserver (override nsStubImageDecoderObserver)
+  NS_IMETHOD OnStartContainer(imgIRequest *aRequest, imgIContainer *aImage);
+  NS_IMETHOD OnDataAvailable(imgIRequest *aRequest, gfxIImageFrame *aFrame,
+                             const nsRect *aRect);
+  NS_IMETHOD OnStopDecode(imgIRequest *aRequest, nsresult status,
+                          const PRUnichar *statusArg);
+  // imgIContainerObserver (override nsStubImageDecoderObserver)
+  NS_IMETHOD FrameChanged(imgIContainer *aContainer, gfxIImageFrame *newframe,
+                          nsRect * dirtyRect);
 
   void SetFrame(nsBulletFrame *frame) { mFrame = frame; }
 
@@ -80,20 +86,12 @@ private:
 };
 
 
-
-
-
-
-nsBulletFrame::nsBulletFrame()
-{
-}
-
 nsBulletFrame::~nsBulletFrame()
 {
 }
 
-NS_IMETHODIMP
-nsBulletFrame::Destroy(nsPresContext* aPresContext)
+void
+nsBulletFrame::Destroy()
 {
   // Stop image loading first
   if (mImageRequest) {
@@ -102,10 +100,10 @@ nsBulletFrame::Destroy(nsPresContext* aPresContext)
   }
 
   if (mListener)
-    NS_REINTERPRET_CAST(nsBulletListener*, mListener.get())->SetFrame(nsnull);
+    reinterpret_cast<nsBulletListener*>(mListener.get())->SetFrame(nsnull);
 
   // Let base class do the rest
-  return nsFrame::Destroy(aPresContext);
+  nsFrame::Destroy();
 }
 
 #ifdef NS_DEBUG
@@ -119,11 +117,23 @@ nsBulletFrame::GetFrameName(nsAString& aResult) const
 nsIAtom*
 nsBulletFrame::GetType() const
 {
-  return nsLayoutAtoms::bulletFrame;
+  return nsGkAtoms::bulletFrame;
+}
+
+PRBool
+nsBulletFrame::IsEmpty()
+{
+  return IsSelfEmpty();
+}
+
+PRBool
+nsBulletFrame::IsSelfEmpty() 
+{
+  return GetStyleList()->mListStyleType == NS_STYLE_LIST_STYLE_NONE;
 }
 
 NS_IMETHODIMP
-nsBulletFrame::DidSetStyleContext(nsPresContext* aPresContext)
+nsBulletFrame::DidSetStyleContext()
 {
   imgIRequest *newRequest = GetStyleList()->mListStyleImage;
 
@@ -173,211 +183,161 @@ nsBulletFrame::DidSetStyleContext(nsPresContext* aPresContext)
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsBulletFrame::Paint(nsPresContext*      aPresContext,
-                     nsIRenderingContext& aRenderingContext,
-                     const nsRect&        aDirtyRect,
-                     nsFramePaintLayer    aWhichLayer,
-                     PRUint32             aFlags)
+class nsDisplayBullet : public nsDisplayItem {
+public:
+  nsDisplayBullet(nsBulletFrame* aFrame) : nsDisplayItem(aFrame) {
+    MOZ_COUNT_CTOR(nsDisplayBullet);
+  }
+#ifdef NS_BUILD_REFCNT_LOGGING
+  virtual ~nsDisplayBullet() {
+    MOZ_COUNT_DTOR(nsDisplayBullet);
+  }
+#endif
+
+  virtual nsIFrame* HitTest(nsDisplayListBuilder* aBuilder, nsPoint aPt,
+                            HitTestState* aState) { return mFrame; }
+  virtual void Paint(nsDisplayListBuilder* aBuilder, nsIRenderingContext* aCtx,
+     const nsRect& aDirtyRect);
+  NS_DISPLAY_DECL_NAME("Bullet")
+};
+
+void nsDisplayBullet::Paint(nsDisplayListBuilder* aBuilder,
+     nsIRenderingContext* aCtx, const nsRect& aDirtyRect)
 {
-  if (NS_FRAME_PAINT_LAYER_FOREGROUND != aWhichLayer) {
+  static_cast<nsBulletFrame*>(mFrame)->
+    PaintBullet(*aCtx, aBuilder->ToReferenceFrame(mFrame), aDirtyRect);
+}
+
+NS_IMETHODIMP
+nsBulletFrame::BuildDisplayList(nsDisplayListBuilder*   aBuilder,
+                                const nsRect&           aDirtyRect,
+                                const nsDisplayListSet& aLists)
+{
+  if (!IsVisibleForPainting(aBuilder))
     return NS_OK;
-  }
 
-  PRBool isVisible;
-  if (NS_SUCCEEDED(IsVisibleForPainting(aPresContext, aRenderingContext, PR_TRUE, &isVisible)) && isVisible) {
-    const nsStyleList* myList = GetStyleList();
-    PRUint8 listStyleType = myList->mListStyleType;
+  DO_GLOBAL_REFLOW_COUNT_DSP("nsBulletFrame");
+  
+  return aLists.Content()->AppendNewToTop(new (aBuilder) nsDisplayBullet(this));
+}
 
-    if (myList->mListStyleImage && mImageRequest) {
-      PRUint32 status;
-      mImageRequest->GetImageStatus(&status);
-      if (status & imgIRequest::STATUS_LOAD_COMPLETE &&
-          !(status & imgIRequest::STATUS_ERROR)) {
-        nsCOMPtr<imgIContainer> imageCon;
-        mImageRequest->GetImage(getter_AddRefs(imageCon));
-        if (imageCon) {
-          nsRect innerArea(0, 0,
-                           mRect.width - (mPadding.left + mPadding.right),
-                           mRect.height - (mPadding.top + mPadding.bottom));
-          nsRect dest(mPadding.left, mPadding.top, innerArea.width, innerArea.height);
-          aRenderingContext.DrawImage(imageCon, innerArea, dest);
-          return NS_OK;
-        }
+void
+nsBulletFrame::PaintBullet(nsIRenderingContext& aRenderingContext, nsPoint aPt,
+                           const nsRect& aDirtyRect)
+{
+  const nsStyleList* myList = GetStyleList();
+  PRUint8 listStyleType = myList->mListStyleType;
+
+  if (myList->mListStyleImage && mImageRequest) {
+    PRUint32 status;
+    mImageRequest->GetImageStatus(&status);
+    if (status & imgIRequest::STATUS_LOAD_COMPLETE &&
+        !(status & imgIRequest::STATUS_ERROR)) {
+      nsCOMPtr<imgIContainer> imageCon;
+      mImageRequest->GetImage(getter_AddRefs(imageCon));
+      if (imageCon) {
+        nsRect dest(mPadding.left, mPadding.top,
+                    mRect.width - (mPadding.left + mPadding.right),
+                    mRect.height - (mPadding.top + mPadding.bottom));
+        nsLayoutUtils::DrawImage(&aRenderingContext, imageCon,
+                                 dest + aPt, aDirtyRect);
+        return;
       }
     }
-
-    const nsStyleFont* myFont = GetStyleFont();
-    const nsStyleColor* myColor = GetStyleColor();
-
-    nsCOMPtr<nsIFontMetrics> fm;
-    aRenderingContext.SetColor(myColor->mColor);
-
-#ifdef IBMBIDI
-    nsCharType charType = eCharType_LeftToRight;
-    PRUint8 level = 0;
-    PRBool isBidiSystem = PR_FALSE;
-    const nsStyleVisibility* vis = GetStyleVisibility();
-    PRUint32 hints = 0;
-#endif // IBMBIDI
-
-    nsAutoString text;
-    switch (listStyleType) {
-    case NS_STYLE_LIST_STYLE_NONE:
-      break;
-
-    default:
-    case NS_STYLE_LIST_STYLE_DISC:
-      aRenderingContext.FillEllipse(mPadding.left, mPadding.top,
-                                    mRect.width - (mPadding.left + mPadding.right),
-                                    mRect.height - (mPadding.top + mPadding.bottom));
-      break;
-
-    case NS_STYLE_LIST_STYLE_CIRCLE:
-      aRenderingContext.DrawEllipse(mPadding.left, mPadding.top,
-                                    mRect.width - (mPadding.left + mPadding.right),
-                                    mRect.height - (mPadding.top + mPadding.bottom));
-      break;
-
-    case NS_STYLE_LIST_STYLE_SQUARE:
-      aRenderingContext.FillRect(mPadding.left, mPadding.top,
-                                 mRect.width - (mPadding.left + mPadding.right),
-                                 mRect.height - (mPadding.top + mPadding.bottom));
-      break;
-
-    case NS_STYLE_LIST_STYLE_DECIMAL:
-    case NS_STYLE_LIST_STYLE_OLD_DECIMAL:
-    case NS_STYLE_LIST_STYLE_DECIMAL_LEADING_ZERO:
-#ifdef IBMBIDI
-      GetListItemText(*myList, text);
-      charType = eCharType_EuropeanNumber;
-      break;
-
-    case NS_STYLE_LIST_STYLE_MOZ_ARABIC_INDIC:
-      if (GetListItemText(*myList, text))
-        charType = eCharType_ArabicNumber;
-      else
-        charType = eCharType_EuropeanNumber;
-      break;
-
-    case NS_STYLE_LIST_STYLE_HEBREW:
-      aRenderingContext.GetHints(hints);
-      isBidiSystem = (hints & NS_RENDERING_HINT_BIDI_REORDERING);
-      if (!isBidiSystem) {
-        if (GetListItemText(*myList, text)) {
-          charType = eCharType_RightToLeft;
-          level = 1;
-        } else {
-          charType = eCharType_EuropeanNumber;
-        }
-
-        if (NS_STYLE_DIRECTION_RTL == vis->mDirection) {
-          text.Cut(0, 1);
-          text.AppendLiteral(".");
-        }
-        break;
-      }
-      // else fall through
-#endif // IBMBIDI
-
-    case NS_STYLE_LIST_STYLE_LOWER_ROMAN:
-    case NS_STYLE_LIST_STYLE_UPPER_ROMAN:
-    case NS_STYLE_LIST_STYLE_LOWER_ALPHA:
-    case NS_STYLE_LIST_STYLE_UPPER_ALPHA:
-    case NS_STYLE_LIST_STYLE_OLD_LOWER_ROMAN:
-    case NS_STYLE_LIST_STYLE_OLD_UPPER_ROMAN:
-    case NS_STYLE_LIST_STYLE_OLD_LOWER_ALPHA:
-    case NS_STYLE_LIST_STYLE_OLD_UPPER_ALPHA:
-    case NS_STYLE_LIST_STYLE_LOWER_GREEK:
-#ifndef IBMBIDI
-    case NS_STYLE_LIST_STYLE_HEBREW:
-#endif
-    case NS_STYLE_LIST_STYLE_ARMENIAN:
-    case NS_STYLE_LIST_STYLE_GEORGIAN:
-    case NS_STYLE_LIST_STYLE_CJK_IDEOGRAPHIC:
-    case NS_STYLE_LIST_STYLE_HIRAGANA:
-    case NS_STYLE_LIST_STYLE_KATAKANA:
-    case NS_STYLE_LIST_STYLE_HIRAGANA_IROHA:
-    case NS_STYLE_LIST_STYLE_KATAKANA_IROHA:
-    case NS_STYLE_LIST_STYLE_MOZ_SIMP_CHINESE_INFORMAL: 
-    case NS_STYLE_LIST_STYLE_MOZ_SIMP_CHINESE_FORMAL: 
-    case NS_STYLE_LIST_STYLE_MOZ_TRAD_CHINESE_INFORMAL: 
-    case NS_STYLE_LIST_STYLE_MOZ_TRAD_CHINESE_FORMAL: 
-    case NS_STYLE_LIST_STYLE_MOZ_JAPANESE_INFORMAL: 
-    case NS_STYLE_LIST_STYLE_MOZ_JAPANESE_FORMAL: 
-    case NS_STYLE_LIST_STYLE_MOZ_CJK_HEAVENLY_STEM:
-    case NS_STYLE_LIST_STYLE_MOZ_CJK_EARTHLY_BRANCH:
-#ifndef IBMBIDI
-    case NS_STYLE_LIST_STYLE_MOZ_ARABIC_INDIC:
-#endif
-    case NS_STYLE_LIST_STYLE_MOZ_PERSIAN:
-    case NS_STYLE_LIST_STYLE_MOZ_URDU:
-    case NS_STYLE_LIST_STYLE_MOZ_DEVANAGARI:
-    case NS_STYLE_LIST_STYLE_MOZ_GURMUKHI:
-    case NS_STYLE_LIST_STYLE_MOZ_GUJARATI:
-    case NS_STYLE_LIST_STYLE_MOZ_ORIYA:
-    case NS_STYLE_LIST_STYLE_MOZ_KANNADA:
-    case NS_STYLE_LIST_STYLE_MOZ_MALAYALAM:
-    case NS_STYLE_LIST_STYLE_MOZ_BENGALI:
-    case NS_STYLE_LIST_STYLE_MOZ_TAMIL:
-    case NS_STYLE_LIST_STYLE_MOZ_TELUGU:
-    case NS_STYLE_LIST_STYLE_MOZ_THAI:
-    case NS_STYLE_LIST_STYLE_MOZ_LAO:
-    case NS_STYLE_LIST_STYLE_MOZ_MYANMAR:
-    case NS_STYLE_LIST_STYLE_MOZ_KHMER:
-    case NS_STYLE_LIST_STYLE_MOZ_HANGUL:
-    case NS_STYLE_LIST_STYLE_MOZ_HANGUL_CONSONANT:
-    case NS_STYLE_LIST_STYLE_MOZ_ETHIOPIC_HALEHAME:
-    case NS_STYLE_LIST_STYLE_MOZ_ETHIOPIC_NUMERIC:
-    case NS_STYLE_LIST_STYLE_MOZ_ETHIOPIC_HALEHAME_AM:
-    case NS_STYLE_LIST_STYLE_MOZ_ETHIOPIC_HALEHAME_TI_ER:
-    case NS_STYLE_LIST_STYLE_MOZ_ETHIOPIC_HALEHAME_TI_ET:
-      fm = aPresContext->GetMetricsFor(myFont->mFont);
-#ifdef IBMBIDI
-      // If we can't render our numeral using the chars in the numbering
-      // system, we'll be using "decimal"...
-      PRBool usedChars =
-#endif // IBMBIDI
-      GetListItemText(*myList, text);
-#ifdef IBMBIDI
-      if (!usedChars)
-        charType = eCharType_EuropeanNumber;
-#endif
-      aRenderingContext.SetFont(fm);
-      nscoord ascent;
-      fm->GetMaxAscent(ascent);
-      aRenderingContext.DrawString(text, mPadding.left, mPadding.top + ascent);
-      break;
-    }
-#ifdef IBMBIDI
-    if (charType != eCharType_LeftToRight) {
-      fm = aPresContext->GetMetricsFor(myFont->mFont);
-      aRenderingContext.SetFont(fm);
-      nscoord ascent;
-      fm->GetMaxAscent(ascent);
-
-      nsBidiPresUtils* bidiUtils = aPresContext->GetBidiUtils();
-      if (bidiUtils) {
-        const PRUnichar* buffer = text.get();
-        PRInt32 textLength = text.Length();
-        if (eCharType_RightToLeft == charType) {
-          bidiUtils->FormatUnicodeText(aPresContext, (PRUnichar*)buffer, textLength,
-                                       charType, level, PR_FALSE);
-        }
-        else {
-//Mohamed
-          aRenderingContext.GetHints(hints);
-          isBidiSystem = (hints & NS_RENDERING_HINT_ARABIC_SHAPING);
-          bidiUtils->FormatUnicodeText(aPresContext, (PRUnichar*)buffer, textLength,
-                                       charType, level, isBidiSystem);//Mohamed
-        }
-      }
-      aRenderingContext.DrawString(text, mPadding.left, mPadding.top + ascent);
-    }   
-#endif // IBMBIDI
   }
-  DO_GLOBAL_REFLOW_COUNT_DSP("nsBulletFrame", &aRenderingContext);
-  return NS_OK;
+
+  const nsStyleColor* myColor = GetStyleColor();
+
+  nsCOMPtr<nsIFontMetrics> fm;
+  aRenderingContext.SetColor(myColor->mColor);
+
+  mTextIsRTL = PR_FALSE;
+
+  nsAutoString text;
+  switch (listStyleType) {
+  case NS_STYLE_LIST_STYLE_NONE:
+    break;
+
+  default:
+  case NS_STYLE_LIST_STYLE_DISC:
+    aRenderingContext.FillEllipse(mPadding.left + aPt.x, mPadding.top + aPt.y,
+                                  mRect.width - (mPadding.left + mPadding.right),
+                                  mRect.height - (mPadding.top + mPadding.bottom));
+    break;
+
+  case NS_STYLE_LIST_STYLE_CIRCLE:
+    aRenderingContext.DrawEllipse(mPadding.left + aPt.x, mPadding.top + aPt.y,
+                                  mRect.width - (mPadding.left + mPadding.right),
+                                  mRect.height - (mPadding.top + mPadding.bottom));
+    break;
+
+  case NS_STYLE_LIST_STYLE_SQUARE:
+    aRenderingContext.FillRect(mPadding.left + aPt.x, mPadding.top + aPt.y,
+                               mRect.width - (mPadding.left + mPadding.right),
+                               mRect.height - (mPadding.top + mPadding.bottom));
+    break;
+
+  case NS_STYLE_LIST_STYLE_DECIMAL:
+  case NS_STYLE_LIST_STYLE_OLD_DECIMAL:
+  case NS_STYLE_LIST_STYLE_DECIMAL_LEADING_ZERO:
+  case NS_STYLE_LIST_STYLE_LOWER_ROMAN:
+  case NS_STYLE_LIST_STYLE_UPPER_ROMAN:
+  case NS_STYLE_LIST_STYLE_LOWER_ALPHA:
+  case NS_STYLE_LIST_STYLE_UPPER_ALPHA:
+  case NS_STYLE_LIST_STYLE_OLD_LOWER_ROMAN:
+  case NS_STYLE_LIST_STYLE_OLD_UPPER_ROMAN:
+  case NS_STYLE_LIST_STYLE_OLD_LOWER_ALPHA:
+  case NS_STYLE_LIST_STYLE_OLD_UPPER_ALPHA:
+  case NS_STYLE_LIST_STYLE_LOWER_GREEK:
+  case NS_STYLE_LIST_STYLE_HEBREW:
+  case NS_STYLE_LIST_STYLE_ARMENIAN:
+  case NS_STYLE_LIST_STYLE_GEORGIAN:
+  case NS_STYLE_LIST_STYLE_CJK_IDEOGRAPHIC:
+  case NS_STYLE_LIST_STYLE_HIRAGANA:
+  case NS_STYLE_LIST_STYLE_KATAKANA:
+  case NS_STYLE_LIST_STYLE_HIRAGANA_IROHA:
+  case NS_STYLE_LIST_STYLE_KATAKANA_IROHA:
+  case NS_STYLE_LIST_STYLE_MOZ_SIMP_CHINESE_INFORMAL: 
+  case NS_STYLE_LIST_STYLE_MOZ_SIMP_CHINESE_FORMAL: 
+  case NS_STYLE_LIST_STYLE_MOZ_TRAD_CHINESE_INFORMAL: 
+  case NS_STYLE_LIST_STYLE_MOZ_TRAD_CHINESE_FORMAL: 
+  case NS_STYLE_LIST_STYLE_MOZ_JAPANESE_INFORMAL: 
+  case NS_STYLE_LIST_STYLE_MOZ_JAPANESE_FORMAL: 
+  case NS_STYLE_LIST_STYLE_MOZ_CJK_HEAVENLY_STEM:
+  case NS_STYLE_LIST_STYLE_MOZ_CJK_EARTHLY_BRANCH:
+  case NS_STYLE_LIST_STYLE_MOZ_ARABIC_INDIC:
+  case NS_STYLE_LIST_STYLE_MOZ_PERSIAN:
+  case NS_STYLE_LIST_STYLE_MOZ_URDU:
+  case NS_STYLE_LIST_STYLE_MOZ_DEVANAGARI:
+  case NS_STYLE_LIST_STYLE_MOZ_GURMUKHI:
+  case NS_STYLE_LIST_STYLE_MOZ_GUJARATI:
+  case NS_STYLE_LIST_STYLE_MOZ_ORIYA:
+  case NS_STYLE_LIST_STYLE_MOZ_KANNADA:
+  case NS_STYLE_LIST_STYLE_MOZ_MALAYALAM:
+  case NS_STYLE_LIST_STYLE_MOZ_BENGALI:
+  case NS_STYLE_LIST_STYLE_MOZ_TAMIL:
+  case NS_STYLE_LIST_STYLE_MOZ_TELUGU:
+  case NS_STYLE_LIST_STYLE_MOZ_THAI:
+  case NS_STYLE_LIST_STYLE_MOZ_LAO:
+  case NS_STYLE_LIST_STYLE_MOZ_MYANMAR:
+  case NS_STYLE_LIST_STYLE_MOZ_KHMER:
+  case NS_STYLE_LIST_STYLE_MOZ_HANGUL:
+  case NS_STYLE_LIST_STYLE_MOZ_HANGUL_CONSONANT:
+  case NS_STYLE_LIST_STYLE_MOZ_ETHIOPIC_HALEHAME:
+  case NS_STYLE_LIST_STYLE_MOZ_ETHIOPIC_NUMERIC:
+  case NS_STYLE_LIST_STYLE_MOZ_ETHIOPIC_HALEHAME_AM:
+  case NS_STYLE_LIST_STYLE_MOZ_ETHIOPIC_HALEHAME_TI_ER:
+  case NS_STYLE_LIST_STYLE_MOZ_ETHIOPIC_HALEHAME_TI_ET:
+    nsLayoutUtils::GetFontMetricsForFrame(this, getter_AddRefs(fm));
+    GetListItemText(*myList, text);
+    aRenderingContext.SetFont(fm);
+    nscoord ascent;
+    fm->GetMaxAscent(ascent);
+    aRenderingContext.SetTextRunRTL(mTextIsRTL);
+    aRenderingContext.DrawString(text, mPadding.left + aPt.x,
+                                 mPadding.top + aPt.y + ascent);
+    break;
+  }
 }
 
 PRInt32
@@ -396,7 +356,7 @@ nsBulletFrame::SetListItemOrdinal(PRInt32 aNextOrdinal,
     nsGenericHTMLElement *hc =
       nsGenericHTMLElement::FromContent(parentContent);
     if (hc) {
-      const nsAttrValue* attr = hc->GetParsedAttr(nsHTMLAtoms::value);
+      const nsAttrValue* attr = hc->GetParsedAttr(nsGkAtoms::value);
       if (attr && attr->Type() == nsAttrValue::eInteger) {
         // Use ordinal specified by the value attribute
         mOrdinal = attr->GetIntegerValue();
@@ -808,9 +768,7 @@ static const PRBool CJKIdeographicToText(PRInt32 ordinal, nsString& result,
   return PR_TRUE;
 }
 
-#define HEBREW_THROSAND_SEP 0x0020
 #define HEBREW_GERESH       0x05F3
-#define HEBREW_GERSHAYIM    0x05F4
 static const PRUnichar gHebrewDigit[22] = 
 {
 //   1       2       3       4       5       6       7       8       9
@@ -823,73 +781,22 @@ static const PRUnichar gHebrewDigit[22] =
 
 static PRBool HebrewToText(PRInt32 ordinal, nsString& result)
 {
-  if (ordinal < 0) {
+  if (ordinal < 1 || ordinal > 999999) {
     DecimalToText(ordinal, result);
     return PR_FALSE;
   }
-  if (ordinal == 0) {
-    // This one is treated specially
-#ifdef IBMBIDI
-    static const PRUnichar hebrewZero[] = { 0x05D0, 0x05E4, 0x05E1 };
-#else
-    static const PRUnichar hebrewZero[] = { 0x05E1, 0x05E4, 0x05D0 };
-#endif // IBMBIDI
-    result.Append(hebrewZero);
-    return PR_TRUE;
-  }
   PRBool outputSep = PR_FALSE;
-  PRUnichar buf[NUM_BUF_SIZE];
-#ifdef IBMBIDI
-  // Changes: 1) don't reverse the text; 2) don't insert geresh/gershayim.
-  PRInt32 idx = 0;
-#else
-  PRInt32 idx = NUM_BUF_SIZE;
-#endif // IBMBIDI
-  PRUnichar digit;
+  nsAutoString allText, thousandsGroup;
   do {
+    thousandsGroup.Truncate();
     PRInt32 n3 = ordinal % 1000;
-    if(outputSep)
-#ifdef IBMBIDI
-      buf[idx++] = HEBREW_THROSAND_SEP; // output thousand separator
-#else
-      buf[--idx] = HEBREW_THROSAND_SEP; // output thousand separator
-#endif // IBMBIDI
-    outputSep = ( n3 > 0); // request to output thousand separator next time.
-
-    PRInt32 d = 0; // we need to keep track of digit got output per 3 digits,
-    // so we can handle Gershayim and Gersh correctly
-
     // Process digit for 100 - 900
     for(PRInt32 n1 = 400; n1 > 0; )
     {
       if( n3 >= n1)
       {
         n3 -= n1;
-
-        digit = gHebrewDigit[(n1/100)-1+18];
-        if( n3 > 0)
-        {
-#ifdef IBMBIDI
-          buf[idx++] = digit;
-#else
-          buf[--idx] = digit;
-#endif // IBMBIDI
-          ++d;
-        } else { 
-          // if this is the last digit
-#ifdef IBMBIDI
-          buf[idx++] = digit;
-#else
-          if (d > 0)
-          {
-            buf[--idx] = HEBREW_GERSHAYIM; 
-            buf[--idx] = digit;
-          } else {
-            buf[--idx] = digit;    
-            buf[--idx] = HEBREW_GERESH;
-          } // if
-#endif // IBMBIDI
-        } // if
+        thousandsGroup.Append(gHebrewDigit[(n1/100)-1+18]);
       } else {
         n1 -= 100;
       } // if
@@ -905,63 +812,28 @@ static PRBool HebrewToText(PRInt32 ordinal, nsString& result)
         // 15 is represented by 9 and 6, not 10 and 5
         // 16 is represented by 9 and 7, not 10 and 6
         n2 = 9;
-        digit = gHebrewDigit[ n2 - 1];    
+        thousandsGroup.Append(gHebrewDigit[ n2 - 1]);
       } else {
         n2 = n3 - (n3 % 10);
-        digit = gHebrewDigit[(n2/10)-1+9];
+        thousandsGroup.Append(gHebrewDigit[(n2/10)-1+9]);
       } // if
-
       n3 -= n2;
-
-      if( n3  > 0) {
-#ifdef IBMBIDI
-        buf[idx++] = digit;
-#else
-        buf[--idx] = digit;
-#endif // IBMBIDI
-        ++d;
-      } else {
-        // if this is the last digit
-#ifdef IBMBIDI
-        buf[idx++] = digit;
-#else
-        if (d > 0)
-        {
-          buf[--idx] = HEBREW_GERSHAYIM;  
-          buf[--idx] = digit;
-        } else {
-          buf[--idx] = digit;    
-          buf[--idx] = HEBREW_GERESH;
-        } // if
-#endif // IBMBIDI
-      } // if
     } // if
   
     // Process digit for 1 - 9 
     if ( n3 > 0)
-    {
-      digit = gHebrewDigit[n3-1];
-      // must be the last digit
-#ifdef IBMBIDI
-      buf[idx++] = digit;
-#else
-      if (d > 0)
-      {
-        buf[--idx] = HEBREW_GERSHAYIM; 
-        buf[--idx] = digit;
-      } else {
-        buf[--idx] = digit;    
-        buf[--idx] = HEBREW_GERESH;
-      } // if
-#endif // IBMBIDI
-    } // if
+      thousandsGroup.Append(gHebrewDigit[n3-1]);
+    if (outputSep) 
+      thousandsGroup.Append((PRUnichar)HEBREW_GERESH);
+    if (allText.IsEmpty())
+      allText = thousandsGroup;
+    else
+      allText = thousandsGroup + allText;
     ordinal /= 1000;
+    outputSep = PR_TRUE;
   } while (ordinal >= 1);
-#ifdef IBMBIDI
-  result.Append(buf, idx);
-#else
-  result.Append(buf+idx,NUM_BUF_SIZE-idx);
-#endif // IBMBIDI
+
+  result.Append(allText);
   return PR_TRUE;
 }
 
@@ -1349,16 +1221,8 @@ PRBool
 nsBulletFrame::GetListItemText(const nsStyleList& aListStyle,
                                nsString& result)
 {
-#ifdef IBMBIDI
   const nsStyleVisibility* vis = GetStyleVisibility();
 
-  // XXX For some of these systems, "." is wrong!  This should really be
-  // pushed down into the individual cases!
-  if (NS_STYLE_DIRECTION_RTL == vis->mDirection) {
-    result.AppendLiteral(".");
-  }
-#endif // IBMBIDI
-
   NS_ASSERTION(aListStyle.mListStyleType != NS_STYLE_LIST_STYLE_NONE &&
                aListStyle.mListStyleType != NS_STYLE_LIST_STYLE_DISC &&
                aListStyle.mListStyleType != NS_STYLE_LIST_STYLE_CIRCLE &&
@@ -1366,30 +1230,29 @@ nsBulletFrame::GetListItemText(const nsStyleList& aListStyle,
                "we should be using specialized code for these types");
   PRBool success =
     AppendCounterText(aListStyle.mListStyleType, mOrdinal, result);
+  if (success && aListStyle.mListStyleType == NS_STYLE_LIST_STYLE_HEBREW)
+    mTextIsRTL = PR_TRUE;
 
   // XXX For some of these systems, "." is wrong!  This should really be
-  // pushed up into the cases...
-#ifdef IBMBIDI
-  if (NS_STYLE_DIRECTION_RTL != vis->mDirection)
-#endif // IBMBIDI
-  result.AppendLiteral(".");
+  // pushed down into the individual cases!
+  nsString suffix = NS_LITERAL_STRING(".");
+
+  // We're not going to do proper Bidi reordering on the list item marker, but
+  // just display the whole thing as RTL or LTR, so we fake reordering by
+  // appending the suffix to the end of the list item marker if the
+  // directionality of the characters is the same as the style direction or
+  // prepending it to the beginning if they are different.
+  result = (mTextIsRTL == (vis->mDirection == NS_STYLE_DIRECTION_RTL)) ?
+          result + suffix : suffix + result;
   return success;
 }
 
 #define MIN_BULLET_SIZE 1
 
 
-#define MINMAX(_value,_min,_max) \
-    ((_value) < (_min)           \
-     ? (_min)                    \
-     : ((_value) > (_max)        \
-        ? (_max)                 \
-        : (_value)))
-
-
 void
 nsBulletFrame::GetDesiredSize(nsPresContext*  aCX,
-                              const nsHTMLReflowState& aReflowState,
+                              nsIRenderingContext *aRenderingContext,
                               nsHTMLReflowMetrics& aMetrics)
 {
   // Reset our padding.  If we need it, we'll set it below.
@@ -1403,90 +1266,12 @@ nsBulletFrame::GetDesiredSize(nsPresContext*  aCX,
     mImageRequest->GetImageStatus(&status);
     if (status & imgIRequest::STATUS_SIZE_AVAILABLE &&
         !(status & imgIRequest::STATUS_ERROR)) {
-      nscoord widthConstraint = NS_INTRINSICSIZE;
-      nscoord heightConstraint = NS_INTRINSICSIZE;
-      PRBool fixedContentWidth = PR_FALSE;
-      PRBool fixedContentHeight = PR_FALSE;
-
-      nscoord minWidth, maxWidth, minHeight, maxHeight;
-      
-      // Determine whether the image has fixed content width
-      widthConstraint = aReflowState.mComputedWidth;
-      minWidth = aReflowState.mComputedMinWidth;
-      maxWidth = aReflowState.mComputedMaxWidth;
-      if (widthConstraint != NS_INTRINSICSIZE) {
-        fixedContentWidth = PR_TRUE;
-      }
-
-      // Determine whether the image has fixed content height
-      heightConstraint = aReflowState.mComputedHeight;
-      minHeight = aReflowState.mComputedMinHeight;
-      maxHeight = aReflowState.mComputedMaxHeight;
-      if (heightConstraint != NS_UNCONSTRAINEDSIZE) {
-        fixedContentHeight = PR_TRUE;
-      }
-
-      PRBool haveComputedSize = PR_FALSE;
-      PRBool needIntrinsicImageSize = PR_FALSE;
-
-      nscoord newWidth=0, newHeight=0;
-      if (fixedContentWidth) {
-        newWidth = MINMAX(widthConstraint, minWidth, maxWidth);
-        if (fixedContentHeight) {
-          newHeight = MINMAX(heightConstraint, minHeight, maxHeight);
-          haveComputedSize = PR_TRUE;
-        } else {
-          // We have a width, and an auto height. Compute height from
-          // width once we have the intrinsic image size.
-          if (mIntrinsicSize.height != 0) {
-            newHeight = (mIntrinsicSize.height * newWidth) / mIntrinsicSize.width;
-            haveComputedSize = PR_TRUE;
-          } else {
-            newHeight = 0;
-            needIntrinsicImageSize = PR_TRUE;
-          }
-        }
-      } else if (fixedContentHeight) {
-        // We have a height, and an auto width. Compute width from height
-        // once we have the intrinsic image size.
-        newHeight = MINMAX(heightConstraint, minHeight, maxHeight);
-        if (mIntrinsicSize.width != 0) {
-          newWidth = (mIntrinsicSize.width * newHeight) / mIntrinsicSize.height;
-          haveComputedSize = PR_TRUE;
-        } else {
-          newWidth = 0;
-          needIntrinsicImageSize = PR_TRUE;
-        }
-      } else {
-        // auto size the image
-        if (mIntrinsicSize.width == 0 && mIntrinsicSize.height == 0)
-          needIntrinsicImageSize = PR_TRUE;
-        else
-          haveComputedSize = PR_TRUE;
-
-        newWidth = mIntrinsicSize.width;
-        newHeight = mIntrinsicSize.height;
-      }
-
-      mComputedSize.width = newWidth;
-      mComputedSize.height = newHeight;
-
-#if 0 // don't do scaled images in bullets
-      if (mComputedSize == mIntrinsicSize) {
-        mTransform.SetToIdentity();
-      } else {
-        if (mComputedSize.width != 0 && mComputedSize.height != 0) {
-          mTransform.SetToScale(float(mIntrinsicSize.width) / float(mComputedSize.width),
-                                float(mIntrinsicSize.height) / float(mComputedSize.height));
-        }
-      }
-#endif
+      // auto size the image
+      mComputedSize.width = mIntrinsicSize.width;
+      mComputedSize.height = mIntrinsicSize.height;
 
       aMetrics.width = mComputedSize.width;
-      aMetrics.height = mComputedSize.height;
-
-      aMetrics.ascent = aMetrics.height;
-      aMetrics.descent = 0;
+      aMetrics.ascent = aMetrics.height = mComputedSize.height;
 
       return;
     }
@@ -1500,38 +1285,26 @@ nsBulletFrame::GetDesiredSize(nsPresContext*  aCX,
   // match the image size).
   mIntrinsicSize.SizeTo(0, 0);
 
-  const nsStyleFont* myFont = GetStyleFont();
-  nsCOMPtr<nsIFontMetrics> fm = aCX->GetMetricsFor(myFont->mFont);
+  nsCOMPtr<nsIFontMetrics> fm;
+  nsLayoutUtils::GetFontMetricsForFrame(this, getter_AddRefs(fm));
   nscoord bulletSize;
-  float p2t;
-  float t2p;
 
   nsAutoString text;
   switch (myList->mListStyleType) {
     case NS_STYLE_LIST_STYLE_NONE:
       aMetrics.width = 0;
-      aMetrics.height = 0;
-      aMetrics.ascent = 0;
-      aMetrics.descent = 0;
+      aMetrics.ascent = aMetrics.height = 0;
       break;
 
     case NS_STYLE_LIST_STYLE_DISC:
     case NS_STYLE_LIST_STYLE_CIRCLE:
     case NS_STYLE_LIST_STYLE_SQUARE:
-      t2p = aCX->TwipsToPixels();
       fm->GetMaxAscent(ascent);
-      bulletSize = NSTwipsToIntPixels(
-        (nscoord)NSToIntRound(0.8f * (float(ascent) / 2.0f)), t2p);
-      if (bulletSize < MIN_BULLET_SIZE) {
-        bulletSize = MIN_BULLET_SIZE;
-      }
-      p2t = aCX->PixelsToTwips();
-      bulletSize = NSIntPixelsToTwips(bulletSize, p2t);
-      mPadding.bottom = NSIntPixelsToTwips((nscoord) NSToIntRound((float)ascent / (8.0f * p2t)),p2t);
+      bulletSize = PR_MAX(nsPresContext::CSSPixelsToAppUnits(MIN_BULLET_SIZE),
+                          NSToCoordRound(0.8f * (float(ascent) / 2.0f)));
+      mPadding.bottom = NSToCoordRound(float(ascent) / 8.0f);
       aMetrics.width = mPadding.right + bulletSize;
-      aMetrics.height = mPadding.bottom + bulletSize;
-      aMetrics.ascent = mPadding.bottom + bulletSize;
-      aMetrics.descent = 0;
+      aMetrics.ascent = aMetrics.height = mPadding.bottom + bulletSize;
       break;
 
     default:
@@ -1588,11 +1361,10 @@ nsBulletFrame::GetDesiredSize(nsPresContext*  aCX,
     case NS_STYLE_LIST_STYLE_MOZ_ETHIOPIC_HALEHAME_TI_ET:
       GetListItemText(*myList, text);
       fm->GetHeight(aMetrics.height);
-      aReflowState.rendContext->SetFont(fm);
-      aReflowState.rendContext->GetWidth(text, aMetrics.width);
+      aRenderingContext->SetFont(fm);
+      aMetrics.width = nsLayoutUtils::GetStringWidth(this, aRenderingContext, text.get(), text.Length());
       aMetrics.width += mPadding.right;
       fm->GetMaxAscent(aMetrics.ascent);
-      fm->GetMaxDescent(aMetrics.descent);
       break;
   }
 }
@@ -1603,11 +1375,11 @@ nsBulletFrame::Reflow(nsPresContext* aPresContext,
                       const nsHTMLReflowState& aReflowState,
                       nsReflowStatus& aStatus)
 {
-  DO_GLOBAL_REFLOW_COUNT("nsBulletFrame", aReflowState.reason);
+  DO_GLOBAL_REFLOW_COUNT("nsBulletFrame");
   DISPLAY_REFLOW(aPresContext, this, aReflowState, aMetrics, aStatus);
 
   // Get the base size
-  GetDesiredSize(aPresContext, aReflowState, aMetrics);
+  GetDesiredSize(aPresContext, aReflowState.rendContext, aMetrics);
 
   // Add in the border and padding; split the top/bottom between the
   // ascent and descent to make things look nice
@@ -1615,16 +1387,36 @@ nsBulletFrame::Reflow(nsPresContext* aPresContext,
   aMetrics.width += borderPadding.left + borderPadding.right;
   aMetrics.height += borderPadding.top + borderPadding.bottom;
   aMetrics.ascent += borderPadding.top;
-  aMetrics.descent += borderPadding.bottom;
 
-  if (aMetrics.mComputeMEW) {
-    aMetrics.mMaxElementWidth = aMetrics.width;
-  }
+  // XXX this is a bit of a hack, we're assuming that no glyphs used for bullets
+  // overflow their font-boxes. It'll do for now; to fix it for real, we really
+  // should rewrite all the text-handling code here to use gfxTextRun (bug
+  // 397294).
+  aMetrics.mOverflowArea.SetRect(0, 0, aMetrics.width, aMetrics.height);
+  
   aStatus = NS_FRAME_COMPLETE;
   NS_FRAME_SET_TRUNCATION(aStatus, aReflowState, aMetrics);
   return NS_OK;
 }
 
+/* virtual */ nscoord
+nsBulletFrame::GetMinWidth(nsIRenderingContext *aRenderingContext)
+{
+  nsHTMLReflowMetrics metrics;
+  DISPLAY_MIN_WIDTH(this, metrics.width);
+  GetDesiredSize(PresContext(), aRenderingContext, metrics);
+  return metrics.width;
+}
+
+/* virtual */ nscoord
+nsBulletFrame::GetPrefWidth(nsIRenderingContext *aRenderingContext)
+{
+  nsHTMLReflowMetrics metrics;
+  DISPLAY_PREF_WIDTH(this, metrics.width);
+  GetDesiredSize(PresContext(), aRenderingContext, metrics);
+  return metrics.width;
+}
+
 
 NS_IMETHODIMP nsBulletFrame::OnStartContainer(imgIRequest *aRequest,
                                               imgIContainer *aImage)
@@ -1642,11 +1434,10 @@ NS_IMETHODIMP nsBulletFrame::OnStartContainer(imgIRequest *aRequest,
   aImage->GetWidth(&w);
   aImage->GetHeight(&h);
 
-  float p2t;
-  nsPresContext* presContext = GetPresContext();
-  p2t = presContext->PixelsToTwips();
+  nsPresContext* presContext = PresContext();
 
-  nsSize newsize(NSIntPixelsToTwips(w, p2t), NSIntPixelsToTwips(h, p2t));
+  nsSize newsize(nsPresContext::CSSPixelsToAppUnits(w),
+                 nsPresContext::CSSPixelsToAppUnits(h));
 
   if (mIntrinsicSize != newsize) {
     mIntrinsicSize = newsize;
@@ -1655,11 +1446,8 @@ NS_IMETHODIMP nsBulletFrame::OnStartContainer(imgIRequest *aRequest,
     // a reflow of the bullet frame.
     nsIPresShell *shell = presContext->GetPresShell();
     if (shell) {
-      NS_ASSERTION(mParent, "No parent to pass the reflow request up to.");
-      if (mParent) {
-        mState |= NS_FRAME_IS_DIRTY;
-        mParent->ReflowDirtyChild(shell, this);
-      }
+      shell->FrameNeedsReflow(this, nsIPresShell::eStyleChange,
+                              NS_FRAME_IS_DIRTY);
     }
   }
 
@@ -1752,11 +1540,6 @@ nsBulletListener::~nsBulletListener()
 {
 }
 
-NS_IMETHODIMP nsBulletListener::OnStartDecode(imgIRequest *aRequest)
-{
-  return NS_OK;
-}
-
 NS_IMETHODIMP nsBulletListener::OnStartContainer(imgIRequest *aRequest,
                                                  imgIContainer *aImage)
 {
@@ -1766,12 +1549,6 @@ NS_IMETHODIMP nsBulletListener::OnStartContainer(imgIRequest *aRequest,
   return mFrame->OnStartContainer(aRequest, aImage);
 }
 
-NS_IMETHODIMP nsBulletListener::OnStartFrame(imgIRequest *aRequest,
-                                             gfxIImageFrame *aFrame)
-{
-  return NS_OK;
-}
-
 NS_IMETHODIMP nsBulletListener::OnDataAvailable(imgIRequest *aRequest,
                                                 gfxIImageFrame *aFrame,
                                                 const nsRect *aRect)
@@ -1782,18 +1559,6 @@ NS_IMETHODIMP nsBulletListener::OnDataAvailable(imgIRequest *aRequest,
   return mFrame->OnDataAvailable(aRequest, aFrame, aRect);
 }
 
-NS_IMETHODIMP nsBulletListener::OnStopFrame(imgIRequest *aRequest,
-                                            gfxIImageFrame *aFrame)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsBulletListener::OnStopContainer(imgIRequest *aRequest,
-                                                imgIContainer *aImage)
-{
-  return NS_OK;
-}
-
 NS_IMETHODIMP nsBulletListener::OnStopDecode(imgIRequest *aRequest,
                                              nsresult status,
                                              const PRUnichar *statusArg)
